/*
 * QUSOFT MICROSYSTMES
 * Power Basic
 * Copyright 2004 Frdric Brown
 */

/*
 *  This file is part of Power Basic library.
 *
 *  Power Basic library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Power Basic library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Power Basic library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "Functions.h"

#if USES_NCR || USES_NPR
long factorial(long n) {
	long fac = 1;
	long i;

	for (i = 2; i <= n; i++) {
		fac *= i;
	}

	return fac;
}
#endif

#ifdef USES_AUGMENT
Variant AUGMENT ( Variant l1, Variant l2 ) {
	int l1c = l1.array->cNum;
	int l2c = l2.array->cNum;
	int l1r = l1.array->rNum;
	int l2r = l2.array->rNum;
	int n;
	int i;

	/* A: l1 is a list or matrice */

	if (l1.array->rNum == 1) {
		/* A: Both l1 and l2 are lists */

		/* Reallocation of list 1 */
		l1.array->cNum += l2c;
		l1.array->ptr[0] = (Variant *)realloc(l1.array->ptr[0], sizeof(Variant) * l1.array->cNum);

		/* A: Enough memory */

		/* Concatenation of list 2 */

		for (i = l1c; i < l1c + l2c; i++) {
			l1.array->ptr[0][i] = l2.array->ptr[0][i - l1c];
		}
	}
	else {
		/* A: Both l1 and l2 are matrices */

		/* Reallocation of matrice 1 */
			/* l1.array->cNum += l2c;
			for (i = l1c; i < l1.array->cNum; i++) {
				l1.array->ptr[i] = (Variant *)realloc(l1.array->ptr[i], sizeof(Variant) * l1.array->cNum);
			} */

		redimArray(l1.array, l1r, l1c + l2c);

		/* A: Enough memory */

		/* Concatenation of matrice 2 */
		for (i = l1c; i < l1.array->cNum; i++) {
			for (n = 0; n < l1r; n++) {
				l1.array->ptr[n][i] = l2.array->ptr[n][i-l1c];
			}
		}
	}

	return l1;
}
#endif

#ifdef USES_EXPR
double EXPR (Variant v) {

	if (v.type == VARIANT_NUMERIC) {
		return v.num;
	}

	/* A: v is a String variant */
	return strToFloat(v.str);
}
#endif

#ifdef USES_CHAR
Variant CHAR (short n) {
	Variant v;

	v.type = VARIANT_STRING;
	v.str[0] = (char)n;
	v.str[1] = '\0';

	return v;
}
#endif

#ifdef USES_COLDIM
long COLDIM (Variant v) {
	return v.array->cNum;
}
#endif

#ifdef USES_COLNORM
long COLNORM (Variant v) {
	double max = DBL_MIN;
	double sum;
	short i;
	short n;

	/* A: v is a matrix */

	for (i = 0; i < v.array->cNum; i++) {
		sum = 0;
		for (n = 0; n < v.array->rNum; n++) {
			/* A: v.array->ptr[n][i] is numeric */
			sum += abs(v.array->ptr[n][i].num);
		}
		if (sum > max) {
			max = sum;
		}
	}

	return max;
}
#endif

#ifdef USES_CUMSUM
Variant CUMSUM (Variant m) {
	short i;
	short n;
	double sum;

	if (m.array->rNum == 1) {
		/* A: m is a list */
		sum = 0;
		for (i = 0; i < m.array->cNum; i++) {
			/* A: m.array->ptr[0][i] is a numeric */
			sum += m.array->ptr[0][i].num;
			m.array->ptr[0][i].num = sum;
		}
	}
	else {
		/* A: m is a matrix */
		for (i = 0; i < m.array->cNum; i++) {
			sum = 0;
			for (n = 0; n < m.array->rNum; n++) {
				/* A: m.array->ptr[n][i] is a numeric */
				sum += m.array->ptr[n][i].num;
				m.array->ptr[n][i].num = sum;
			}
		}
	}

	return m;
}
#endif

#ifdef USES_DIM
long DIM (Variant v) {
	if (v.type == VARIANT_ARRAY) {
		return v.array->rNum * v.array->cNum;
	}
	else {
		/* A: v is a string */
		return strlen(v.str);
	}
}
#endif

#ifdef USES_DOTP
double DOTP (Variant v1, Variant v2) {
	short i;
	double sum = 0;

	/* A: v1 and v2 are vectors of same size */

	for (i = 0; i < v1.array->cNum; i++) {
		sum += v1.array->ptr[0][i].num * v2.array->ptr[0][i].num;
	}

	return sum;
}
#endif

#ifdef USES_FORMAT
/* ! FIXME - This function is incomplete ! */
Variant FORMAT(Variant e, Variant t) {
	Variant s;

	/* A: t is a string */

	if (cmpstri(t.str, "F") == 0) {
		/* A: e is numeric */
		sprintf(s.str, "%f", e.num);
	}
	else if (cmpstri(t.str, "S") == 0) {
		/* A: e is numeric */
		sprintf(s.str, "%.-1E", e.num);
	}

	return s;
}
#endif

#ifdef USES_FPART
Variant FPART(Variant v) {
	short i;
	short n;

	if (v.type == VARIANT_NUMERIC) {
		v.num = v.num - (((long)v.num) - v.num);
	}
	else {
		/* A: m1 is a matrix */

		for (i = 0; i < v.array->rNum; i++) {
			for (n = 0; n < v.array->cNum; n++) {
				/* A: v.array->ptr[i][n] is numeric */
				v.array->ptr[i][n].num = (((long)v.array->ptr[i][n].num) - v.array->ptr[i][n].num);
			}
		}
	}

	return v;
}
#endif

#ifdef USES_GCD
long findPcd (long a, long b) {
	long maxim;
	long minim;
	long pcd;

	if ((a < 1) || (b < 1)) {
		/* a or b is invalid input */
		return -1;
	}

	if (a > b) {
		maxim = a;
		pcd = b;
	}
	else {
		maxim = b;
		pcd = a;
	}

	while ((minim % pcd) || (maxim % pcd)) pcd--; /* In the worst case, will quits when pcd == 1 */

	return pcd;
}

Variant GCD(Variant v1, Variant v2) {
	short i;
	short n;

	if (v1.type == VARIANT_NUMERIC) {
		v1.num = findPcd(v1.num, v2.num);
	}
	else {
		/* v1 and v2 are both matrix; v2 is as large as v1 */
		for (i = 0; i < v1.array->rNum; i++) {
			for (n = 0; n < v1.array->cNum; n++) {
				/* A: v1.array->ptr[i][n] is numeric */
				v1.array->ptr[i][n].num = findPcd(v1.array->ptr[i][n].num, v2.array->ptr[i][n].num);
			}
		}
	}

	return v1;
}
#endif

#ifdef USES_GETFOLD
Variant GETFOLD() {
	char buffer[VARIANT_STRING_SIZE];

	FolderGetCur(buffer);

	return newVariantStr(buffer);
}
#endif

#ifdef USES_IDENTITY
Variant IDENTITY(Variant m, short size) {
	short i;

	redimArray(m.array, size, size);

	/* A: Enough memory */

	for (i = 0; i < size; i++) {
		m.array->ptr[i][i] = newVariantNum(1);
	}
}
#endif

#ifdef USES_INSTRING
long INSTRING(Variant str, Variant sub, long b) {
	return (strstr(&str.str[b], sub.str) - str.str);
}
#endif

#ifdef USES_INTDIV
Variant INTDIV (Variant m1, Variant m2) {
	short i;
	short n;

	if (m1.type == VARIANT_NUMERIC) {
		/* A: m2 is numeric */
		m1.num = (long)(m1.num / m2.num);
	}
	else {
		/* A: m1 and m2 are both matrices */

		for (i = 0; i < m1.array->rNum; i++) {
			for (n = 0; n < m1.array->cNum; n++) {
				m1.array->ptr[i][n].num = (long)(m1.array->ptr[i][n].num / m2.array->ptr[i][n].num);
			}
		}
	}

	return m1;
}
#endif

#ifdef USES_IPART
Variant IPART(Variant v) {
	short i;
	short n;

	if (v.type == VARIANT_NUMERIC) {
		v.num = (long)v.num;
	}
	else {
		/* A: m1 is a matrix */

		for (i = 0; i < v.array->rNum; i++) {
			for (n = 0; n < v.array->cNum; n++) {
				v.array->ptr[i][n].num = (long)v.array->ptr[i][n].num;
			}
		}
	}

	return v;
}
#endif

#ifdef USES_LCM
long findLcm (long a, long b) {
	long maxim;
	long minim;
	long lcm;

	if ((a < 1) || (b < 1)) {
		/* a or b is invalid input */
		return -1;
	}

	if (a > b) {
		maxim = a;
		lcm = a;
	}
	else {
		maxim = b;
		lcm = b;
	}

	while ((lcm % maxim) || (lcm % minim)) lcm++;

	return lcm;
}

Variant LCM(Variant v1, Variant v2) {
	short i;
	short n;

	if (v1.type == VARIANT_NUMERIC) {
		v1.num = findLcm(v1.num, v2.num);
	}
	else {
		/* v1 and v2 are both matrix; v2 is as large as v1 */
		for (i = 0; i < v1.array->rNum; i++) {
			for (n = 0; n < v1.array->cNum; n++) {
				v1.array->ptr[i][n].num = findLcm(v1.array->ptr[i][n].num, v2.array->ptr[i][n].num);
			}
		}
	}

	return v1;
}
#endif

#ifdef USES_LEFT
Variant LEFT(Variant s, long n) {
	short i;

	if (s.type == VARIANT_STRING) {
		s.str[n] = '\0';
	}
	else {
		/* A: s is a list */

		for (i = 0; i < n; i++) {
			s.array->ptr[0][i] = s.array->ptr[0][i+(s.array->cNum-n)];
		}

		redimArray(s.array, 1, n);
	}

	return s;
}
#endif

#if USES_MAT_LIST || USES_MEDIAN
Variant MAT_LIST(Variant m) {
	VariantArray newa;
	short i;
	short n;

	newArray(&newa, 1, m.array->rNum * m.array->cNum);

	/* A: Enough memory */

	for (i = 0; i < m.array->rNum; i++) {
		for (n = 0; n < m.array->cNum; n++) {
			newa.ptr[i][n + (i * m.array->cNum)] = m.array->ptr[i][n];
		}
	}

	freeArray(m.array);

	m.array->ptr = newa.ptr;
	m.array->cNum = newa.cNum;
	m.array->rNum = newa.rNum;

	return m;
}
#endif

#ifdef USES_MAX
/*Variant MAX(Variant v1, Variant v2) {
	short i;
	short n;

	if (v1.type == VARIANT_NUMERIC) {
		// A: v2 is numeric
		v1.num = max(v1.num, v2.num);
	}
	else {
		// A: v1 and v2 are both matrix

		for (i = 0; i < v1.array->rNum; i++) {
			for (n = 0; n < v1.array->cNum; n++) {
				// A: m1.array->ptr[i][n] and m2.array->ptr[i][n] are numeric
				v1.array->ptr[i][n].num = max(v1.array->ptr[i][n].num, v2.array->ptr[i][n].num);
			}
		}
	}

	return v1;
}*/
#endif

#ifdef USES_MEAN
double MEAN(Variant m) {
	short i;
	short n;
	long size;
	double mean = 0;

	/* A: m is an array */
	size = m.array->rNum * m.array->cNum;

	for (i = 0; i < m.array->rNum; i++) {
		for (n = 1; n < m.array->cNum; n++) {
			/* A: m.array->ptr[i][n] is numeric */
			mean += (m.array->ptr[i][n].num / size);
		}
	}

	return mean;
}
#endif

#ifdef USES_MEDIAN
CALLBACK short variant_cmpformedian(const void *a, const void *b)
{
	/* A: Variants a and b are numeric */
  	return fcmp ((*(const Variant*)a).num, (*(const Variant*)b).num);
}

double MEDIAN(Variant m) {
	short i;
	short n;
	VariantArray tmp;
	double med = 0;

	/* A: m is an array */
	newArray(&tmp, 1, m.array->rNum * m.array->cNum);

	/* A: enough memory */

	for (i = 0; i < m.array->rNum; i++) {
		for (n = 0; n < m.array->cNum; n++) {
			tmp.ptr[i][n] = m.array->ptr[i][n];
		}
	}

	MAT_LIST(newVariantArray(&tmp));

	qsort (tmp.ptr[0], tmp.cNum, sizeof (Variant), variant_cmpformedian);

	med = tmp.ptr[0][(tmp.cNum + 1) / 2].num;

	freeArray(&tmp);

	return med;
}
#endif

#ifdef USES_MID
Variant MID(Variant s, long b, long n) {
	short i;

	if (s.type == VARIANT_STRING) {
		memmove(s.str, &s.str[b - 1], sizeof(char) * n);
		s.str[e] = '\0';
	}
	else {
		/* A: s is a list */

		for (i = 0; i < n; i++) {
			s.array->ptr[0][i] = s.array->ptr[0][i+(b-1)];
		}

		redimArray(s.array, 1, n);
	}

	return s;
}
#endif

#ifdef USES_MROW
Variant MROW(double expr, Variant m, short numL) {
	short i;

	/* A: m is a matrix */

	for (i = 0; i < m.array->cNum; i++) {
		/* A: m.array->ptr[numL][i] is numeric */
		m.array->ptr[numL][i].num = expr * m.array->ptr[numL][i].num;
	}

	return m;
}
#endif

#ifdef USES_MROWADD
Variant MROWADD(double expr, Variant m, short numL1, short numL2) {
	short i;

	/* A: m is a matrix */

	for (i = 0; i < m.array->cNum; i++) {
		/* A: m.array->ptr[numL1][i] and m.array->ptr[numL2][i] are numeric */
		m.array->ptr[numL2][i].num = expr * m.array->ptr[numL1][i].num + m.array->ptr[numL2][i].num;
	}

	return m;
}
#endif

#ifdef USES_NCR
long NCR(long n, long r) {
	return factorial(n) / (factorial(n-r)*factorial(r));
}
#endif

#ifdef USES_NORM
double NORM(Variant m) {
	double norm = 0;
	short i;
	short n;

	for (i = 0; i < m.array->rNum; i++) {
		for (n = 0; n < m.array->cNum; n++) {
			/* A: m.array->ptr[i][n] is numeric */
			norm += pow(m.array->ptr[i][n].num, 2);
		}
	}

	/* A: m is a matrix */

	return sqrt(norm);
}
#endif

#ifdef USES_NPR
long NPR(long n, long r) {
	return factorial(n) / factorial(n-r);
}
#endif

#ifdef USES_ORD
short ORD(Variant s) {
	/* A: s is a string */
	return s.str[0];
}
#endif

#ifdef USES_PRODUCT
double product(Variant m) {
	short i;
	short n;
	double prod = 1;

	/* A: m is a list or a matrix */
	for (i = 0; i < m.array->rNum; i++) {
		for (n = 0; n < m.array->cNum; n++) {
			/* A: m.array->ptr[i][n] is numeric */
			prod *= pow(m.array->ptr[i][n].num, 2);
		}
	}

	return prod;
}
#endif

#ifdef USES_PXLTEST
Variant PXLTEST(Variant ly, Variant lx) {
	short i;

	if (lx.type == VARIANT_NUMERIC) {
		lx.num = GetPix(lx.num, ly.num);
	}
	else {
		/* A: lx and ly are both matrixes ; ly is at least as wide as lx */
		for (i = 0; i < lx.array->cNum; i++) {
			/* A: lx.array->ptr[0][i] and ly.array->ptr[0][i] are numeric */
			lx.array->ptr[0][i].num = GetPix(lx.array->ptr[0][i].num, ly.array->ptr[0][i].num);
		}
	}

	return ly;
}
#endif

#ifdef USES_RAND
double RAND(short n) {
	if (n) {
		return random(n) + 1;
	}
	else {
		return ((double)rand()) / RAND_MAX + 1;
	}
}
#endif

#ifdef USES_RANDMAT
Variant RANDMAT(Variant m, short nbL, short nbC) {
	short i;
	short n;
	short rnd;

	redimArray(m.array, nbL, nbC);

	for (i = 0; i < m.array->rNum; i++) {
		for (n = 0; n < m.array->cNum; n++) {
			rnd = random(9) + 1;
			if (random(2)) {
				rnd = -rnd;
			}

			m.array->ptr[i][n] = newVariantNum(rnd);
		}
	}

	return m;
}
#endif

#ifdef USES_REMAIN
Variant REMAIN (Variant m1, Variant m2) {
	short i;
	short n;

	if (m1.type == VARIANT_NUMERIC) {
		/* A: m2 is numeric */
		m1.num = (long)m1.num % (long)m2.num;
	}
	else {
		/* A: m1 and m2 are both matrices */

		for (i = 0; i < m1.array->rNum; i++) {
			for (n = 0; n < m1.array->cNum; n++) {
				((long)m1.array->ptr[i][n].num) %= (long)m2.array->ptr[i][n].num;
			}
		}
	}

	return m1;
}
#endif

#ifdef USES_RIGHT
Variant RIGHT(Variant s, long n) {
	char buffer[VARIANT_STRING_SIZE];
	short i;

	if (s.type == VARIANT_STRING) {
		strcpy(buffer, &s.str[strlen(s.str) - n]);
		s = newVariantStr(buffer);
	}
	else {
		/* A: s is a list */

		redimArray(s.array, 1, n);
	}

	return s;
}
#endif

#ifdef USES_STOMAT
Variant STOMAT (Variant m1, Variant m2) {
	short i;
	short n;

	/* A: m1 and m2 are both matrices */
	redimArray(m1.array, m2.array->rNum, m2.array->cNum) ;

	/* A: Enough memory */

	for (i = 0; i < m1.array->rNum; i++) {
		for (n = 0; n < m1.array->cNum; n++) {
			m1.array->ptr[i][n] = m2.array->ptr[i][n];
		}
	}

	return m1;
}
#endif
