/*
 * QUSOFT MICROSYSTMES
 * Power Basic
 * Copyright 2004 Frdric Brown
 */

/*
 *  This file is part of Power Basic library.
 *
 *  Power Basic library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Power Basic library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Power Basic library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "Procedures.h"

#if USES_STOPIC || USES_NEWPIC
void saveBMP(BITMAP *bmp, Variant v) {
	//Code from Qs CLib BMPGET function
	short taille = bmp->NumRows*((bmp->NumCols+7)>>3)+4;//The length of the bitmap
	SYM_ENTRY *sym_entry;//A pointer on the TIOS VAT entry
	char var[VARIANT_STRING_SIZE];
	char *vat;//Will contain the name of the variable, in TIOS string format

	sym_entry = DerefSym(SymAdd(toVATBuffer(var, v.str))) ;//Creates a new TIOS entry and assigns it to sym_entry

	vat = (unsigned char*)HeapDeref(sym_entry->handle = HeapAlloc(taille+3)) ;//Assigns the adress of the TIOS variable to
	//Writes the dimensions of the PIC to the PIC header
	vat[0] =  (taille + 1) / 256;
	vat[1] =  (taille + 1) % 256;
	memcpy(vat+2, bmp, taille) ;//Copy the bitmap data to the TIOS variable
	vat[taille+2] = 0xDF ;//Ends the TIOS variable by the PIC_VAR tag (DF)
}
#endif

#if USES_PUTPIC || USES_CYCLEPIC
void PUTPIC (Variant v, short x, short y, short attr) {
	char buffer[VARIANT_STRING_SIZE];	
	SYM_ENTRY *sym_entry = SymFindPtr (toVATBuffer(buffer, v.str), 0);
	
		if (!sym_entry) return;
		if (peek (HToESI (sym_entry->handle)) != PIC_TAG) return;
		
		BitmapPut (x, y, HeapDeref (sym_entry->handle) + 2, ScrRect, attr);
}
#endif

#ifdef USES_SHELL
void SHELL (Variant v) {
	execString(v.str);
}
#endif

#ifdef USES_FILL
void FILL (Variant e, Variant m) {
	short int i;
	short int n;
	
	
	for (i = 0; i < m.array->rNum; i++) {
		for (n = 0; n < m.array->cNum; n++) {
			m.array->ptr[i][n] = e;
		}
	}
}
#endif

#ifdef USES_COPYVAR
void COPYVAR(Variant v1, Variant v2) {
	char buffer1[VARIANT_STRING_SIZE];
	char buffer2[VARIANT_STRING_SIZE];
	
	cmd_copyvar(toVATBuffer(buffer1, v1.str), toVATBuffer(buffer2, v2.str));
}
#endif

#ifdef USES_DELFOLD
void DELFOLD (Variant v) {
	char buffer[VARIANT_STRING_SIZE];
	
	FolderDel(toVATBuffer(buffer, v.str));	
}
#endif

#ifdef USES_DELVAR
void DELVAR (Variant v) {
	char buffer[VARIANT_STRING_SIZE];
	
	SymDel(toVATBuffer(buffer, v.str));	
}
#endif

#ifdef USES_CYCLEPIC
void CYCLEPIC (Variant v, short nbImg, double del, short nbC, short dir) {
	char buffer[VARIANT_STRING_SIZE];
	char buffer2[VARIANT_STRING_SIZE];
	
	short i;
	short n;
	short b = 1;
	short e = nbImg;
	
	if (dir == -1) {
		b = nbImg;
		e = 1;
	}
	
	for (i = 0; i < nbC; i++) {
		for (n = b; n != e; n += dir) {
			strcpy(buffer, v.str);
			sprintf(buffer2, "%d", n);
			strcat(buffer, buffer2);
			
			PUTPIC(newVariantStr(buffer), 0, 0, A_REPLACE);
			
			OSFreeTimer (USER_TIMER);
			OSRegisterTimer (USER_TIMER, del*20);
			while (!OSTimerExpired (USER_TIMER));
			
		}
	}
}
#endif

#ifdef USES_EXEC
void EXEC (Variant v) {
	command_Exec(v.str);
}
#endif

#ifdef USES_GET
void GET (Variant v) {
	char buffer[VARIANT_STRING_SIZE];
	
	cmd_get(toVATBuffer(buffer, v.str));	
}
#endif

#ifdef USES_GETCALC
void GETCALC (Variant v) {
	char buffer[VARIANT_STRING_SIZE];
	
	cmd_getcalc(toVATBuffer(buffer, v.str));	
}
#endif

#if USES_LOCK || USES_UNLOCK
void setLocked (Variant v, short b) {
	char buffer[VARIANT_STRING_SIZE];
	SYM_ENTRY *SymPtr = SymFindPtr (toVATBuffer(buffer, v.str), 0);
	
	if (!SymPtr) return;
	
	SymPtr->flags.bits.locked = b;
}
#endif

#ifdef USES_MOVEVAR
void MOVEVAR (Variant name, Variant src, Variant dest) {
	char bufferN[VARIANT_STRING_SIZE];
	char bufferS[VARIANT_STRING_SIZE];
	char bufferD[VARIANT_STRING_SIZE];
	
	cmd_movevar (toVATBuffer(bufferN, name.str), toVATBuffer(bufferS, src.str), toVATBuffer(bufferD, dest.str));
}
#endif

#ifdef USES_NEWFOLD
void NEWFOLD (Variant v) {
	char buffer[VARIANT_STRING_SIZE];
	
	cmd_newfold (toVATBuffer(buffer, v.str)); 
}
#endif

#ifdef USES_NEWPIC
void NEWPIC (Variant m, Variant v, short maxX, short maxY) {
	void *virtual = malloc (LCD_SIZE);
	short i;
	short xMax;
	short yMax;
	char buffer [BITMAP_HDR_SIZE + LCD_SIZE];
	
	memset(virtual, 0, LCD_SIZE);
	
	PortSet (virtual, 239, 127); // redirect drawing routines to buffer
	
	xMax = yMax = 0;
	
	for (i = 0; i < m.array->cNum; i++) {
		if (m.array->ptr[0][i].num > xMax) {
			xMax = m.array->ptr[0][i].num;
		}
		if (m.array->ptr[1][i].num > yMax) {
			yMax = m.array->ptr[1][i].num;
		}
		DrawPix(m.array->ptr[0][i].num, m.array->ptr[1][i].num, A_NORMAL);
	}
	
	if (maxX > 0) {
		xMax = maxX;
		yMax = maxY;
	}
	
	
	BitmapGet (&(SCR_RECT){{0, 0, xMax, yMax}}, buffer);
	
	saveBMP((BITMAP*)buffer, v);
	
	PortRestore();	
	
	free(virtual);
}
#endif

#if USES_PXLCHG || USES_PXLOFF || USES_PXLOFF
void PXLSET (Variant x, Variant y, short o) {
	short i;
	
	if (x.type == VARIANT_NUMERIC) {
		/* A: x and y are both numeric variant */
		DrawPix(x.num, y.num, A_REVERSE);
		return;
	}
	
	/* A: x and y are both arrays; y is at least as wide as x */
	for (i = 0; i < x.array->cNum; i++) {
		DrawPix(x.array->ptr[0][i].num, y.array->ptr[0][i].num, o);
	}
}
#endif

#ifdef USES_PXLTEXT
void PXLTEXT (Variant s, short y, short x) {
	DrawStr(x, y, s.str, A_NORMAL);
}
#endif

#ifdef USES_RCLGDB
void RCLGDB(Variant v) {
	char buffer[VARIANT_STRING_SIZE];
	
	cmd_rclgdb (toVATBuffer(buffer, v.str)); 
}
#endif

#ifdef USES_STOGDB
void STOGDB(Variant v) {
	char buffer[VARIANT_STRING_SIZE];
	
	cmd_stogdb (toVATBuffer(buffer, v.str)); 
}
#endif

#ifdef USES_RENAME
void RENAME(Variant v1, Variant v2) {
	char buffer1[VARIANT_STRING_SIZE];
	char buffer2[VARIANT_STRING_SIZE];
	
	cmd_copyvar(toVATBuffer(buffer1, v1.str), toVATBuffer(buffer2, v2.str));
}
#endif

#ifdef USES_SEND
void SEND (Variant v) {
	char buffer[VARIANT_STRING_SIZE];
	
	cmd_send(toVATBuffer(buffer, v.str));	
}
#endif

#ifdef USES_SENDCALC
void SENDCALC (Variant v) {
	char buffer[VARIANT_STRING_SIZE];
	
	cmd_sendcalc(toVATBuffer(buffer, v.str));	
}
#endif

#ifdef USES_SORTA
CALLBACK short variant_cmpa(const void *a, const void *b)
{
	/* A: Variants a and b are numeric */
  	return fcmp ((*(const Variant*)a).num, (*(const Variant*)b).num);
}


void SORTA (Variant v) {
	qsort (v.array->ptr[0], v.array->cNum, sizeof (Variant), variant_cmpa);
}
#endif

#ifdef USES_SORTD

CALLBACK short variant_cmpb(const void *a, const void *b)
{
	/* A: Variants a and b are numeric */
  	return fcmp ((*(const Variant*)b).num, (*(const Variant*)a).num);
}

void SORTD (Variant v) {
	qsort (v.array->ptr[0], v.array->cNum, sizeof (Variant), variant_cmpb);
}
#endif

#ifdef USES_STOPIC
void STOPIC (Variant v, short y, short x, short w, short h) {
	char buffer [BITMAP_HDR_SIZE + LCD_SIZE];

	BitmapGet (&(SCR_RECT){{x, y, x + w - 1, y + h - 1}}, buffer);
	
	saveBMP((BITMAP*)buffer, v);
}
#endif

#ifdef USES_TEXT
void TEXT (Variant message) {
	/* A: message is a string */
	
	DlgMessage (NULL, message.str, BT_OK, BT_NONE);
}
#endif
