/*
 * QUSOFT MICROSYSTMES
 * Power Basic
 * Copyright 2004 Frdric Brown
 */

/*
 *  This file is part of Power Basic library.
 *
 *  Power Basic library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Power Basic library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Power Basic library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* This header defines the Variant type */

#ifndef __VARIANT_HEADER__
#define __VARIANT_HEADER__

#include <tigcclib.h>

#ifndef VARIANT_STRING_SIZE
	#define VARIANT_STRING_SIZE 256
#endif

typedef enum {VARIANT_UNDEFINED, VARIANT_NUMERIC, VARIANT_STRING, VARIANT_POINTER, VARIANT_ARRAY} Variant_Type;

struct Variant_Struct;

struct VariantArray_Struct;

typedef struct Variant_Struct {
	union {
		double num;
		char str[VARIANT_STRING_SIZE];
		struct Variant_Struct * ptr;
		void * voidp;
		struct VariantArray_Struct * array;
	};
	Variant_Type type;
} Variant;

typedef struct VariantArray_Struct {
	struct Variant_Struct ** ptr;
	short int rNum;
	short int cNum;
} VariantArray ;

Variant newVariantArray(VariantArray * array) ;

Variant newVariantStr(char * str) ;

Variant newVariantNum(double num) ;

Variant newVariantPtr(Variant * ptr) ;

void newArray(VariantArray * a, short r, short c) ;
void redimArray(VariantArray * a, short r, short c) ;
void freeArray(VariantArray * a) ;

Variant asVariantString(Variant v) ;

double strToFloat(char * str) ;

Variant asVariantNum(Variant v) ;

Variant asVariantPtr(Variant v) ;

Variant concatVariantStr(Variant v1, Variant v2) ;

double variantCompareTo(Variant v1, Variant v2) ;

char getFontWidth();

char getFontHeight();

Variant fetchVariantFromArg() ;

void command_Exec (char * str) ;

void execString (char * str) ;

void exportVariant(Variant *v, char * varName) ;

void importVariant(Variant *v, char * varName) ;

void returnVariant(Variant v) ;

void initVariantType() ;

void getInput(Variant *v);

char * toVATBuffer(char * buffer, char * str) ;

#endif
